/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

### File Specific:
   main-programmer: Silvan Nellen
   co-programmer: ...
*/

#include "util/loading/factory.h"
#include "action_trigger.h"
#include "debug.h"
#include "event_handler.h"

ObjectListDefinition(ActionTrigger);
CREATE_FACTORY(ActionTrigger);

CREATE_SCRIPTABLE_CLASS(ActionTrigger,
            // Coordinates
                        addMethod("setAbsCoor", Executor3<PNode, lua_State*,float,float,float>(&PNode::setAbsCoor))
                            ->addMethod("getAbsCoorX", Executor0ret<PNode, lua_State*, float>(&PNode::getAbsCoorX))
                            ->addMethod("getAbsCoorY", Executor0ret<PNode, lua_State*, float>(&PNode::getAbsCoorY))
                            ->addMethod("getAbsCoorZ", Executor0ret<PNode, lua_State*, float>(&PNode::getAbsCoorZ))
            //Properties
                            ->addMethod("setName", Executor1<BaseObject, lua_State*, const std::string&>(&BaseObject::setName))
                            ->addMethod("setTarget", Executor1<ActionTrigger, lua_State*, const std::string&>(&ActionTrigger::setTarget))
                            ->addMethod("setTriggerParent", Executor1<ScriptTrigger, lua_State*, const std::string&>(&ScriptTrigger::setTriggerParent))
                            ->addMethod("setInvert", Executor1<ActionTrigger, lua_State*, bool>(&ActionTrigger::setInvert))
                            ->addMethod("setRadius", Executor1<ActionTrigger, lua_State*, float>(&ActionTrigger::setRadius))
                            ->addMethod("setScript", Executor1<ScriptTrigger, lua_State*, const std::string&>(&ScriptTrigger::setScript))
                            ->addMethod("setFunction", Executor1<ScriptTrigger, lua_State*, const std::string&>(&ScriptTrigger::setFunction))
                            ->addMethod("setDebugDraw", Executor1<ScriptTrigger, lua_State*, bool>(&ScriptTrigger::setDebugDraw))
                            ->addMethod("setAddToScript", Executor1<ScriptTrigger, lua_State*, bool>(&ScriptTrigger::setAddToScript))
                       );


/**
 * Constructs a new ActionTrigger.
 * @param root the xml element to load the parameters from.
 *
 */
ActionTrigger::ActionTrigger(const TiXmlElement* root)
{
  this->registerObject(this, ActionTrigger::_objectList);
  this->subscribeEvent(ES_GAME, KeyMapper::PEV_ACTION);

  radius = 10;
  invert = false;
  actionScheduled = false;
 
  if(root != NULL)
  {
    loadParams(root);

    if(addToScript && scriptIsOk)
    {
      script->addObject( "ActionTrigger", this->getName());
    }

  }

}

/**
 * Deletes the ActionTrigger.
 *
 */
ActionTrigger::~ActionTrigger()
{

}

/**
 * Reads the values from the tml element and sets them.
 * @param root the xml element to load the parameters from.
 *
 */
void ActionTrigger::loadParams(const TiXmlElement* root)
{

  ScriptTrigger ::loadParams(root);

 
  LoadParam(root, "radius", this, ActionTrigger, setRadius)
      .describe("the fileName of the script, that should be triggered by this script trigger")
      .defaultValues(0);
  LoadParam(root, "worldentity", this, ActionTrigger, setTarget)
      .describe("The name of the target as it is in the *.oxw file")
      .defaultValues("");
  LoadParam(root, "invert", this, ActionTrigger, setInvert)
      .describe("")
      .defaultValues(false);
 
}


/**
 * Sets the target(a world entity) of the ActionTrigger. If the distance between the target and this trigger is smaller than the radius, the script gets triggered.
 * @param target The worldentity that the script supervises.
 */
void ActionTrigger::setTarget(const std::string& target)
{

  WorldEntity* targetEntity = WorldEntity::objectList().getObject(target);
  if (targetEntity != NULL)
  {
    this->setTarget(targetEntity);
  }
  else
  {
    PRINTF(2)("ERROR SCRTIPTTRIGGER : Target %s for %s::%s does not Exist\n", target.c_str(), this->getClassCName(), this->getCName());
  }
}

void ActionTrigger::process( const Event &event)
{
 if (event.type == KeyMapper::PEV_ACTION && event.bPressed)
  actionScheduled = true;
}

void ActionTrigger::tick( float timestep )
{
  if( scriptFinished || !actionScheduled ) return;

  if( this->target != NULL)
  {
    if( !invert && this->distance(target) < radius && actionScheduled)
    {
      executeScriptFunction(timestep);
      scriptCalled = true;
    }
    else if( invert && this->distance(target) > radius  && actionScheduled)
    {
      executeScriptFunction(timestep);
      scriptCalled = true;
    }
  }

  actionScheduled = false;
}



